@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,6 +36,8 @@
 package net.sourceforge.plantuml;
 
 import java.awt.Font;
+import java.io.IOException;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -46,6 +48,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
 
+import net.sourceforge.plantuml.command.BlocLines;
 import net.sourceforge.plantuml.command.regex.Matcher2;
 import net.sourceforge.plantuml.command.regex.MyPattern;
 import net.sourceforge.plantuml.command.regex.Pattern2;
@@ -61,7 +64,16 @@ import net.sourceforge.plantuml.graphic.HtmlColorUtils;
 import net.sourceforge.plantuml.graphic.IHtmlColorSet;
 import net.sourceforge.plantuml.graphic.SkinParameter;
 import net.sourceforge.plantuml.graphic.color.Colors;
+import net.sourceforge.plantuml.skin.ActorStyle;
 import net.sourceforge.plantuml.skin.ArrowDirection;
+import net.sourceforge.plantuml.skin.Padder;
+import net.sourceforge.plantuml.sprite.Sprite;
+import net.sourceforge.plantuml.sprite.SpriteImage;
+import net.sourceforge.plantuml.style.FromSkinparamToStyle;
+import net.sourceforge.plantuml.style.Style;
+import net.sourceforge.plantuml.style.StyleBuilder;
+import net.sourceforge.plantuml.style.StyleLoader;
+import net.sourceforge.plantuml.svek.ConditionEndStyle;
 import net.sourceforge.plantuml.svek.ConditionStyle;
 import net.sourceforge.plantuml.svek.PackageStyle;
 import net.sourceforge.plantuml.ugraphic.ColorMapper;
@@ -71,36 +83,122 @@ import net.sourceforge.plantuml.ugraphic.ColorMapperReverse;
 import net.sourceforge.plantuml.ugraphic.ColorOrder;
 import net.sourceforge.plantuml.ugraphic.UFont;
 import net.sourceforge.plantuml.ugraphic.UStroke;
-import net.sourceforge.plantuml.ugraphic.sprite.Sprite;
-import net.sourceforge.plantuml.ugraphic.sprite.SpriteImage;
 
 public class SkinParam implements ISkinParam {
 
+	// private String skin = "debug.skin";
+
+	private String skin = "plantuml.skin";
+
+	private SkinParam(UmlDiagramType type) {
+		USE_STYLE2.set(false);
+		this.type = type;
+		if (type == UmlDiagramType.MINDMAP) {
+			USE_STYLE2.set(true);
+		}
+		if (type == UmlDiagramType.WBS) {
+			USE_STYLE2.set(true);
+		}
+		if (type == UmlDiagramType.SEQUENCE) {
+			// skin = "debug.skin";
+			// USE_STYLE2.set(true);
+		}
+		// if (type == UmlDiagramType.ACTIVITY) {
+		// // skin = "debug.skin";
+		// USE_STYLE2.set(true);
+		// }
+	}
+
+	private StyleBuilder styleBuilder;
+
+	public StyleBuilder getCurrentStyleBuilder() {
+		if (styleBuilder == null && SkinParam.USE_STYLES()) {
+			try {
+				this.styleBuilder = getCurrentStyleBuilderInternal();
+			} catch (IOException e) {
+				e.printStackTrace();
+			}
+		}
+		return styleBuilder;
+	}
+
+	public void muteStyle(Style modifiedStyle) {
+		if (SkinParam.USE_STYLES()) {
+			styleBuilder = getCurrentStyleBuilder().muteStyle(modifiedStyle);
+		}
+	}
+
+	public String getDefaultSkin() {
+		return skin;
+	}
+
+	public void setDefaultSkin(String newSkin) {
+		this.skin = newSkin;
+	}
+
+	public StyleBuilder getCurrentStyleBuilderInternal() throws IOException {
+		final StyleLoader tmp = new StyleLoader(this);
+		StyleBuilder result = tmp.loadSkin(this.getDefaultSkin());
+		if (result == null) {
+			result = tmp.loadSkin("plantuml.skin");
+		}
+
+		return result;
+	}
+
+	private static ThreadLocal<Boolean> USE_STYLE2 = new ThreadLocal<Boolean>();
+
+	static public boolean USE_STYLES() {
+		final Boolean result = USE_STYLE2.get();
+		if (result == null) {
+			return false;
+		}
+		return result;
+	}
+
 	private static final String stereoPatternString = "\\<\\<(.*?)\\>\\>";
 	private static final Pattern2 stereoPattern = MyPattern.cmpile(stereoPatternString);
 
 	private final Map<String, String> params = new HashMap<String, String>();
 	private Rankdir rankdir = Rankdir.TOP_TO_BOTTOM;
-	private String dotExecutable;
 	private final UmlDiagramType type;
+	private boolean useVizJs;
 
-	public String getDotExecutable() {
-		return dotExecutable;
+	public void copyAllFrom(ISkinSimple other) {
+		this.params.putAll(other.values());
 	}
 
-	public void setDotExecutable(String dotExecutable) {
-		Log.info("Overwritting dot in skinparam " + dotExecutable);
-		this.dotExecutable = dotExecutable;
+	public Map<String, String> values() {
+		return Collections.unmodifiableMap(params);
 	}
 
 	public void setParam(String key, String value) {
 		for (String key2 : cleanForKey(key)) {
 			params.put(key2, StringUtils.trin(value));
+			if (key2.startsWith("usebetastyle")) {
+				USE_STYLE2.set("true".equalsIgnoreCase(value));
+			}
+			if (USE_STYLES()) {
+				final FromSkinparamToStyle convertor = new FromSkinparamToStyle(key2, value, getCurrentStyleBuilder());
+				for (Style style : convertor.getStyles()) {
+					muteStyle(style);
+				}
+			}
+		}
+		if ("style".equalsIgnoreCase(key) && "strictuml".equalsIgnoreCase(value)) {
+			if (USE_STYLES()) {
+				final InputStream internalIs = StyleLoader.class.getResourceAsStream("/skin/strictuml.skin");
+				final StyleBuilder styleBuilder = this.getCurrentStyleBuilder();
+				try {
+					final BlocLines lines = BlocLines.load(internalIs, null);
+					for (Style modifiedStyle : StyleLoader.getDeclaredStyles(lines, styleBuilder)) {
+						this.muteStyle(modifiedStyle);
+					}
+				} catch (IOException e) {
+					e.printStackTrace();
+				}
+			}
 		}
-	}
-
-	private SkinParam(UmlDiagramType type) {
-		this.type = type;
 	}
 
 	public static SkinParam create(UmlDiagramType type) {
@@ -221,7 +319,7 @@ public class SkinParam implements ISkinParam {
 		if (stereotype == null) {
 			throw new IllegalArgumentException();
 		}
-		final String value2 = getValue("spotchar" + stereotype.getLabel(false));
+		final String value2 = getValue("spotchar" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 		if (value2 != null && value2.length() > 0) {
 			return value2.charAt(0);
 		}
@@ -231,7 +329,7 @@ public class SkinParam implements ISkinParam {
 	public Colors getColors(ColorParam param, Stereotype stereotype) {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getValue(param.name() + "color" + stereotype.getLabel(false));
+			final String value2 = getValue(param.name() + "color" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (value2 != null && getIHtmlColorSet().getColorIfValid(value2) != null) {
 				return new Colors(value2, getIHtmlColorSet(), param.getColorType());
 			}
@@ -262,7 +360,8 @@ public class SkinParam implements ISkinParam {
 	private int getFontSize(Stereotype stereotype, FontParam... param) {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getFirstValueNonNullWithSuffix("fontsize" + stereotype.getLabel(false), param);
+			final String value2 = getFirstValueNonNullWithSuffix(
+					"fontsize" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR), param);
 			if (value2 != null && value2.matches("\\d+")) {
 				return Integer.parseInt(value2);
 			}
@@ -280,7 +379,8 @@ public class SkinParam implements ISkinParam {
 	private String getFontFamily(Stereotype stereotype, FontParam... param) {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getFirstValueNonNullWithSuffix("fontname" + stereotype.getLabel(false), param);
+			final String value2 = getFirstValueNonNullWithSuffix(
+					"fontname" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR), param);
 			if (value2 != null) {
 				return StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(value2);
 			}
@@ -303,7 +403,8 @@ public class SkinParam implements ISkinParam {
 		String value = null;
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			value = getFirstValueNonNullWithSuffix("fontcolor" + stereotype.getLabel(false), param);
+			value = getFirstValueNonNullWithSuffix("fontcolor" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR),
+					param);
 		}
 		if (value == null || getIHtmlColorSet().getColorIfValid(value) == null) {
 			value = getFirstValueNonNullWithSuffix("fontcolor", param);
@@ -331,7 +432,8 @@ public class SkinParam implements ISkinParam {
 		String value = null;
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			value = getFirstValueNonNullWithSuffix("fontstyle" + stereotype.getLabel(false), param);
+			value = getFirstValueNonNullWithSuffix("fontstyle" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR),
+					param);
 		}
 		if (value == null) {
 			value = getFirstValueNonNullWithSuffix("fontstyle", param);
@@ -412,6 +514,7 @@ public class SkinParam implements ISkinParam {
 		result.add("Style");
 		result.add("SequenceParticipant");
 		result.add("ConditionStyle");
+		result.add("ConditionEndStyle");
 		result.add("SameClassWidth");
 		result.add("HyperlinkUnderline");
 		result.add("Padding");
@@ -434,6 +537,7 @@ public class SkinParam implements ISkinParam {
 		result.add("SwimlaneWidth");
 		result.add("SwimlaneWrapTitleWidth");
 		result.add("FixCircleLabelOverlapping");
+		result.add("LifelineStrategy");
 
 		for (FontParam p : EnumSet.allOf(FontParam.class)) {
 			final String h = humanName(p.name());
@@ -531,6 +635,8 @@ public class SkinParam implements ISkinParam {
 		final HorizontalAlignment result = HorizontalAlignment.fromString(value);
 		if (result == null && param == AlignmentParam.noteTextAlignment) {
 			return getDefaultTextAlignment(HorizontalAlignment.LEFT);
+		} else if (result == null && param == AlignmentParam.stateMessageAlignment) {
+			return getDefaultTextAlignment(HorizontalAlignment.CENTER);
 		} else if (result == null) {
 			return param.getDefaultValue();
 		}
@@ -546,6 +652,15 @@ public class SkinParam implements ISkinParam {
 		return result;
 	}
 
+	public HorizontalAlignment getStereotypeAlignment() {
+		final String value = getValue("stereotypealignment");
+		final HorizontalAlignment result = HorizontalAlignment.fromString(value);
+		if (result == null) {
+			return HorizontalAlignment.CENTER;
+		}
+		return result;
+	}
+
 	private String getArg(String value, int i) {
 		if (value == null) {
 			return null;
@@ -579,7 +694,7 @@ public class SkinParam implements ISkinParam {
 	public boolean shadowing(Stereotype stereotype) {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getValue("shadowing" + stereotype.getLabel(false));
+			final String value2 = getValue("shadowing" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (value2 != null) {
 				return value2.equalsIgnoreCase("true");
 			}
@@ -600,7 +715,7 @@ public class SkinParam implements ISkinParam {
 	public boolean shadowingForNote(Stereotype stereotype) {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getValue("note" + "shadowing" + stereotype.getLabel(false));
+			final String value2 = getValue("note" + "shadowing" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (value2 != null) {
 				return value2.equalsIgnoreCase("true");
 			}
@@ -619,7 +734,7 @@ public class SkinParam implements ISkinParam {
 		final String name = skinParameter.getUpperCaseName();
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getValue(name + "shadowing" + stereotype.getLabel(false));
+			final String value2 = getValue(name + "shadowing" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (value2 != null) {
 				return value2.equalsIgnoreCase("true");
 			}
@@ -652,6 +767,10 @@ public class SkinParam implements ISkinParam {
 
 	private final Map<String, Sprite> sprites = new HashMap<String, Sprite>();
 
+	public Collection<String> getAllSpriteNames() {
+		return Collections.unmodifiableCollection(new TreeSet<String>(sprites.keySet()));
+	}
+
 	public void addSprite(String name, Sprite sprite) {
 		sprites.put(name, sprite);
 	}
@@ -743,7 +862,7 @@ public class SkinParam implements ISkinParam {
 
 	private Double getCornerInternal(String key, CornerParam param, Stereotype stereotype) {
 		if (stereotype != null) {
-			key += stereotype.getLabel(false);
+			key += stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR);
 		}
 		final String value = getValue(key);
 		if (value != null && value.matches("\\d+")) {
@@ -757,12 +876,14 @@ public class SkinParam implements ISkinParam {
 		if (stereotype != null) {
 			checkStereotype(stereotype);
 
-			final String styleValue = getValue(param.name() + "style" + stereotype.getLabel(false));
+			final String styleValue = getValue(param.name() + "style"
+					+ stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (styleValue != null) {
 				style = LinkStyle.fromString2(styleValue);
 			}
 
-			final String value2 = getValue(param.name() + "thickness" + stereotype.getLabel(false));
+			final String value2 = getValue(param.name() + "thickness"
+					+ stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (value2 != null && value2.matches("[\\d.]+")) {
 				if (style == null) {
 					style = LinkStyle.NORMAL();
@@ -832,6 +953,15 @@ public class SkinParam implements ISkinParam {
 		return p;
 	}
 
+	public ConditionEndStyle getConditionEndStyle() {
+		final String value = getValue("conditionEndStyle");
+		final ConditionEndStyle p = ConditionEndStyle.fromString(value);
+		if (p == null) {
+			return ConditionEndStyle.DIAMOND;
+		}
+		return p;
+	}
+
 	public double minClassWidth() {
 		final String value = getValue("minclasswidth");
 		if (value != null && value.matches("\\d+")) {
@@ -856,7 +986,7 @@ public class SkinParam implements ISkinParam {
 		String value = getValue("activityshape");
 		if (stereotype != null) {
 			checkStereotype(stereotype);
-			final String value2 = getValue("activityshape" + stereotype.getLabel(false));
+			final String value2 = getValue("activityshape" + stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 			if (value2 != null) {
 				value = value2;
 			}
@@ -896,12 +1026,9 @@ public class SkinParam implements ISkinParam {
 		return result;
 	}
 
-	public boolean useGuillemet() {
+	public Guillemet guillemet() {
 		final String value = getValue("guillemet");
-		if ("false".equalsIgnoreCase(value)) {
-			return false;
-		}
-		return true;
+		return Guillemet.GUILLEMET.fromDescription(value);
 	}
 
 	public boolean handwritten() {
@@ -920,6 +1047,14 @@ public class SkinParam implements ISkinParam {
 		return value;
 	}
 
+	public String getPreserveAspectRatio() {
+		final String value = getValue("preserveaspectratio");
+		if (value == null) {
+			return "none";
+		}
+		return value;
+	}
+
 	public String getMonospacedFamily() {
 		final String value = getValue("defaultMonospacedFontName");
 		if (value == null) {
@@ -990,15 +1125,17 @@ public class SkinParam implements ISkinParam {
 	}
 
 	public double getPadding() {
-		final String value = getValue("padding");
-		if (value != null && value.matches("\\d+(\\.\\d+)?")) {
-			return Double.parseDouble(value);
-		}
-		return 0;
+		final String name = "padding";
+		return getAsDouble(name);
 	}
 
 	public double getPadding(PaddingParam param) {
-		final String value = getValue(param.getSkinName());
+		final String name = param.getSkinName();
+		return getAsDouble(name);
+	}
+
+	private double getAsDouble(final String name) {
+		final String value = getValue(name);
 		if (value != null && value.matches("\\d+(\\.\\d+)?")) {
 			return Double.parseDouble(value);
 		}
@@ -1046,4 +1183,35 @@ public class SkinParam implements ISkinParam {
 		return false;
 	}
 
+	public void setUseVizJs(boolean useVizJs) {
+		this.useVizJs = useVizJs;
+	}
+
+	public boolean isUseVizJs() {
+		return useVizJs;
+	}
+
+	public Padder getSequenceDiagramPadder() {
+		final double padding = getAsDouble("SequenceMessagePadding");
+		final double margin = getAsDouble("SequenceMessageMargin");
+		final String borderColor = getValue("SequenceMessageBorderColor");
+		final String backgroundColor = getValue("SequenceMessageBackGroundColor");
+		if (padding == 0 && margin == 0 && borderColor == null && backgroundColor == null) {
+			return Padder.NONE;
+		}
+		final HtmlColor border = getIHtmlColorSet().getColorIfValid(borderColor);
+		final HtmlColor background = getIHtmlColorSet().getColorIfValid(backgroundColor);
+		final double roundCorner = getRoundCorner(CornerParam.DEFAULT, null);
+		return Padder.NONE.withMargin(margin).withPadding(padding).withBackgroundColor(background)
+				.withBorderColor(border).withRoundCorner(roundCorner);
+	}
+
+	public ActorStyle getActorStyle() {
+		final String value = getValue("actorstyle");
+		if ("awesome".equalsIgnoreCase(value)) {
+			return ActorStyle.AWESOME;
+		}
+		return ActorStyle.STICKMAN;
+	}
+
 }
